@@ -271,7 +271,7 @@ static void share_unlock(CURL *handle, curl_lock_data data, void *userptr) {
 
 static void free_auth(auth_token *tok) {
     if (!tok) return;
-    if (pthread_mutex_destroy(&tok->lock)) abort();
+    if (pthread_mutex_destroy(&tok->lock)) return;
     free(tok->path);
     free(tok->token);
     free(tok);
@@ -837,7 +837,7 @@ static ssize_t libcurl_read(hFILE *fpv, void *bufferv, size_t nbytes)
         got = fp->buffer.ptr.rd - buffer;
 
         if (to_skip >= 0) { // Skipping over a small seek
-            if (got < to_skip) { // Need to skip more data
+            if (got <= to_skip) { // Need to skip more data
                 to_skip -= got;
             } else {
                 got -= to_skip;
